package contrib.tapestry.exception.components;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.ILocation;

public abstract class SourceDisplay extends BaseComponent {

   protected ILocation location = null;

   public String getTitle() {
      if (this.getSourceLocation() != null) {
         return this.getSourceLocation().toString();
      }
      return null;
   }

   public void setTitle(String value) {
      // Do nothing
   }

   public abstract String getContentBeforeLine();

   public abstract void setContentBeforeLine(String value);

   public abstract String getContentAfterLine();

   public abstract void setContentAfterLine(String value);

   public abstract String getContentBeforeColumn();

   public abstract void setContentBeforeColumn(String value);

   public abstract String getContentAfterColumn();

   public abstract void setContentAfterColumn(String value);

   public abstract String getContentTargeted();

   public abstract void setContentTargeted(String value);

   public ILocation getSourceLocation() {
      return this.location;
   }

   public void setSourceLocation(ILocation value) {
      this.location = value;
      this.computeContent();
   }

   protected void computeContent() {
      List content = new ArrayList();
      if (this.getSourceLocation() != null) {
         URL url = this.getSourceLocation().getResourceLocation().getResourceURL();
         try {
            BufferedReader reader = new BufferedReader(new InputStreamReader(url.openStream()));
            String line = null;
            while ((line = reader.readLine()) != null) {
               content.add(line);
            }
         } catch (IOException ioe) {
            ioe.printStackTrace();
         }

         int lineNumber = this.getSourceLocation().getLineNumber() - 1;
         int columnNumber = this.getSourceLocation().getColumnNumber() - 1;

         if (lineNumber >= 0) {
            // In constant
            int firstLineNumber = 0; //Math.max(0, lineNumber - 5);
            int lastLineNumber = content.size(); // Math.min(content.size(), lineNumber + 5);

            if (columnNumber >= 0) {
               String line = this.concatenate(content, lineNumber, lineNumber + 1);
               int eof = line.length() - columnNumber;

               this.setContentBeforeLine(this.concatenate(content, firstLineNumber, lineNumber));
               if (eof > 0) {
                  this.setContentBeforeColumn(line.substring(0, columnNumber));
                  this.setContentTargeted(line.substring(columnNumber, columnNumber + 1));
                  this.setContentAfterColumn(line.substring(columnNumber + 1));
               } else {
                  this.setContentBeforeColumn(line);
                  this.setContentTargeted("");
                  this.setContentAfterColumn("");
               }
               this.setContentAfterLine(this.concatenate(content, lineNumber + 1, lastLineNumber));
            } else {
               this.setContentBeforeLine(this.concatenate(content, firstLineNumber, lineNumber));
               this.setContentBeforeColumn("");
               this.setContentTargeted(this.concatenate(content, lineNumber, lineNumber + 1));
               this.setContentAfterColumn("");
               this.setContentAfterLine(this.concatenate(content, lineNumber + 1, lastLineNumber));
            }
         } else {
            this.setContentBeforeLine("");
            this.setContentBeforeColumn("");
            this.setContentTargeted(this.concatenate(content, 0, content.size()));
            this.setContentAfterColumn("");
            this.setContentAfterLine("");
         }
      }
   }

   protected String concatenate(List content, int start, int end) {
      StringBuffer buffer = new StringBuffer();
      for (int i = start; i < end; i++) {
         String line = (String) content.get(i);
         buffer.append(line);
         buffer.append('\n');
      }
      return buffer.toString();
   }
}